@@ -88,7 +88,10 @@ def main(args, pacu_main: "Main"):
     fetch_data = pacu_main.fetch_data
     # End don't modify
 
-    summary_data = {"users_confirmed": 0, "roles_confirmed": 0}
+    summary_data = {"users_confirmed": 0, "roles_confirmed": 0,
+                   "users_unconfirmed": 0, "roles_unconfirmed": 0,
+                   "users_confirmed_perm_count": 0, "roles_confirmed_perm_count": 0,
+                   "users_unconfirmed_perm_count": 0, "roles_unconfirmed_perm_count": 0}
 
     users = []
     roles = []
@@ -285,6 +288,10 @@ def main(args, pacu_main: "Main"):
                 role = parse_attached_policies(client, attached_policies, role)
                 if role["PermissionsConfirmed"]:
                     summary_data["roles_confirmed"] += 1
+                    summary_data["roles_confirmed_perm_count"] += len(role["Permissions"]["Allow"]) + len(role["Permissions"]["Deny"])
+                else:
+                    summary_data["roles_unconfirmed"] += 1
+                    summary_data["roles_unconfirmed_perm_count"] += len(role["Permissions"]["Allow"]) + len(role["Permissions"]["Deny"])
 
                 if args.role_name is None and args.all_roles is False:
                     print("    Confirmed permissions for {}".format(role["RoleName"]))
@@ -476,6 +483,10 @@ def main(args, pacu_main: "Main"):
                 user = parse_attached_policies(client, attached_policies, user)
                 if user["PermissionsConfirmed"]:
                     summary_data["users_confirmed"] += 1
+                    summary_data["users_confirmed_perm_count"] += len(user["Permissions"]["Allow"]) + len(user["Permissions"]["Deny"])
+                else:
+                    summary_data["users_unconfirmed"] += 1
+                    summary_data["users_unconfirmed_perm_count"] += len(user["Permissions"]["Allow"]) + len(user["Permissions"]["Deny"])
 
                 if args.user_name is None and args.all_users is False:
                     print("    Confirmed Permissions for {}".format(user["UserName"]))
@@ -513,22 +524,52 @@ def main(args, pacu_main: "Main"):
 
 
 def summary(data, pacu_main):
+    """Generate module summary text"""
     out = ""
     if not data:
         return "  Unable to find users/roles to enumerate permissions\n"
-    if data["users_confirmed"] == 1 and data.get("single_user"):
-        out += "  Confirmed permissions for user: {}.\n".format(data["single_user"])
+
+    count_max_len = 2 + max(len(str(data["users_confirmed_perm_count"])),
+                            len(str(data["roles_confirmed_perm_count"])),
+                            len(str(data["users_unconfirmed_perm_count"])),
+                            len(str(data["roles_unconfirmed_perm_count"])))
+
+    # Create Confirmed listings
+    if data.get("single_user") and data["users_confirmed"] == 1:
+        out += "{:>{count_max_len}} Confirmed permissions for user: {}.\n".format(data["users_confirmed_perm_count"],
+                                                                                  data["single_user"],
+                                                                                  count_max_len=count_max_len)
     else:
-        out += "  Confirmed permissions for {} user(s).\n".format(
-            data["users_confirmed"]
-        )
+        out += "{:>{count_max_len}} Confirmed permissions for {} user(s).\n".format(data["users_confirmed_perm_count"],
+                                                                                    data["users_confirmed"],
+                                                                                    count_max_len=count_max_len)
 
     if data["roles_confirmed"] == 1:
-        out += "  Confirmed permissions for role: {}.\n".format(data["single_role"])
+        out += "{:>{count_max_len}} Confirmed permissions for role: {}.\n".format(data["roles_confirmed_perm_count"],
+                                                                                  data["single_role"],
+                                                                                  count_max_len=count_max_len)
     else:
-        out += "  Confirmed permissions for {} role(s).\n".format(
-            data["roles_confirmed"]
-        )
+        out += "{:>{count_max_len}} Confirmed permissions for {} role(s).\n".format(data["roles_confirmed_perm_count"],
+                                                                                    data["roles_confirmed"],
+                                                                                    count_max_len=count_max_len)
+    # Create Unconfirmed listings
+    if data.get("single_user") and data["users_unconfirmed"] == 1:
+            out += "{:>{count_max_len}} Unconfirmed permissions for user: {}.\n".format(data["users_unconfirmed_perm_count"],
+                                                                                  data["single_user"],
+                                                                                  count_max_len=count_max_len)
+    else:
+        out += "{:>{count_max_len}} Unconfirmed permissions for {} user(s).\n".format(data["users_unconfirmed_perm_count"],
+                                                                                  data["users_unconfirmed"],
+                                                                                  count_max_len=count_max_len)
+
+    if data["roles_unconfirmed"] == 1:
+        out += "{:>{count_max_len}} Unconfirmed permissions for role: {}.\n".format(data["roles_unconfirmed_perm_count"],
+                                                                                    data["single_role"],
+                                                                                    count_max_len=count_max_len)
+    else:
+        out += "{:>{count_max_len}} Unconfirmed permissions for {} role(s).\n".format(data["roles_unconfirmed_perm_count"],
+                                                                                      data["roles_unconfirmed"],
+                                                                                      count_max_len=count_max_len)
     return out
 
 
